@@ -369,14 +369,14 @@ xsltAddKey(xsltStylesheetPtr style, const xmlChar *name,
     xsltGenericDebug(xsltGenericDebugContext,
 	"   resulting pattern %s\n", pattern);
 #endif
-    key->comp = xmlXPathCompile(pattern);
+    key->comp = xsltXPathCompile(style, pattern);
     if (key->comp == NULL) {
 	xsltTransformError(NULL, style, inst,
 		"xsl:key : XPath pattern compilation failed '%s'\n",
 		         pattern);
 	if (style != NULL) style->errors++;
     }
-    key->usecomp = xmlXPathCompile(use);
+    key->usecomp = xsltXPathCompile(style, use);
     if (key->usecomp == NULL) {
 	xsltTransformError(NULL, style, inst,
 		"xsl:key : XPath pattern compilation failed '%s'\n",
@@ -644,7 +644,30 @@ xsltInitCtxtKey(xsltTransformContextPtr ctxt, xsltDocumentPtr doc,
 		    } else {
 			xmlXPathNodeSetAdd(keylist, nodelist->nodeTab[i]);
 		    }
-		    nodelist->nodeTab[i]->_private = keyd;
+		    switch (nodelist->nodeTab[i]->type) {
+                        case XML_ELEMENT_NODE:
+                        case XML_TEXT_NODE:
+                        case XML_CDATA_SECTION_NODE:
+                        case XML_PI_NODE:
+                        case XML_COMMENT_NODE:
+			    nodelist->nodeTab[i]->psvi = keyd;
+			    break;
+                        case XML_ATTRIBUTE_NODE: {
+			    xmlAttrPtr attr = (xmlAttrPtr) 
+			                      nodelist->nodeTab[i];
+			    attr->psvi = keyd;
+			    break;
+			}
+                        case XML_DOCUMENT_NODE:
+                        case XML_HTML_DOCUMENT_NODE: {
+			    xmlDocPtr kdoc = (xmlDocPtr) 
+			                    nodelist->nodeTab[i];
+			    kdoc->psvi = keyd;
+			    break;
+			}
+			default:
+			    break;
+		    }
 		    xmlFree(str);
 		    str = list[index++];
 		}
