@@ -491,11 +491,12 @@ xsltAttrTemplateProcess(xsltTransformContextPtr ctxt, xmlNodePtr target,
 	if (text != NULL) {
 	    ret->last = ret->children = text;
 	    text->parent = (xmlNodePtr) ret;
-	    if (cur->_private != NULL) {
+	    text->doc = ret->doc;
+	    if (cur->psvi != NULL) {
 		xmlChar *val;
-		val = xsltEvalAVT(ctxt, cur->_private, cur->parent);
+		val = xsltEvalAVT(ctxt, cur->psvi, cur->parent);
 		if (val == NULL) {
-		    text->content = xmlStrdup("runtime error");
+		    text->content = xmlStrdup(BAD_CAST "runtime error");
 		} else {
 		    text->content = val;
 		}
