@@ -372,6 +372,11 @@ xsltNewStylesheet(void) {
     cur->exclPrefixTab = NULL;
     cur->extInfos = NULL;
     cur->extrasNr = 0;
+    cur->dict = xmlDictCreate();
+#ifdef WITH_XSLT_DEBUG
+    xsltGenericDebug(xsltGenericDebugContext,
+                     "creating dictionary for stylesheet\n");
+#endif
 
     xsltInit();
 
@@ -513,10 +518,18 @@ xsltFreeStylesheet(xsltStylesheetPtr sheet)
         xmlFree(sheet->doctypeSystem);
     if (sheet->mediaType != NULL)
         xmlFree(sheet->mediaType);
+    if (sheet->attVTs)
+        xsltFreeAVTList(sheet->attVTs);
 
     if (sheet->imports != NULL)
         xsltFreeStylesheetList(sheet->imports);
 
+#ifdef WITH_XSLT_DEBUG
+    xsltGenericDebug(xsltGenericDebugContext,
+                     "freeing dictionary from stylesheet\n");
+#endif
+    xmlDictFree(sheet->dict);
+
     memset(sheet, -1, sizeof(xsltStylesheet));
     xmlFree(sheet);
 }
@@ -775,8 +788,21 @@ xsltParseStylesheetOutput(xsltStylesheetPtr style, xmlNodePtr cur)
 		if (element == NULL) {
 		    if (style != NULL) style->errors++;
 		} else {
+		    xmlNsPtr ns;
+		    
 		    xmlHashAddEntry2(style->cdataSection, element, URI,
 			             (void *) "cdata");
+		    /*
+		     * if prefix is NULL, we must check whether it's
+		     * necessary to also put in the name of the default
+		     * namespace.
+		     */
+		    if (URI == NULL) {
+		        ns = xmlSearchNs(style->doc, cur, NULL);
+			if (ns != NULL)  
+			    xmlHashAddEntry2(style->cdataSection, element,
+			        ns->href, (void *) "cdata");
+		    }
 		    xmlFree(element);
 		}
             }
@@ -1181,6 +1207,7 @@ xsltPrecomputeStylesheet(xsltStylesheetPtr style, xmlNodePtr cur) {
 		    goto skip_children;
 		}
 	    }
+
 	    /*
 	     * Remove excluded prefixes
 	     */
@@ -1230,6 +1257,7 @@ xsltPrecomputeStylesheet(xsltStylesheetPtr style, xmlNodePtr cur) {
 		    exclPrefixPop(style);
 		goto skip_children;
 	    }
+
 	} else if (cur->type == XML_TEXT_NODE) {
 	    if (IS_BLANK_NODE(cur)) {
 		if (xmlNodeGetSpacePreserve(cur) != 1) {
@@ -1456,12 +1484,24 @@ xsltParseTemplateContent(xsltStylesheetPtr style, xmlNodePtr templ) {
 		delete = cur;
 		goto skip_children;
 	    }
-	} else if ((cur->ns != NULL) && (style->nsDefs != NULL)) {
-	    if (xsltCheckExtPrefix(style, cur->ns->prefix)) {
-		/*
-		 * okay this is an extension element compile it too
-		 */
-		xsltStylePreCompute(style, cur);
+	} else if ((cur->ns != NULL) && (style->nsDefs != NULL) &&
+	           (xsltCheckExtPrefix(style, cur->ns->prefix))) {
+	    /*
+	     * okay this is an extension element compile it too
+	     */
+	    xsltStylePreCompute(style, cur);
+	} else {
+	    /*
+	     * This is an element which will be output as part of the
+	     * template exectution, precompile AVT if found.
+	     */
+	    if (cur->properties != NULL) {
+	        xmlAttrPtr attr = cur->properties;
+
+		while (attr != NULL) {
+		    xsltCompileAttr(style, attr);
+		    attr = attr->next;
+		}
 	    }
 	}
 
@@ -1858,10 +1898,19 @@ xsltParseStylesheetTop(xsltStylesheetPtr style, xmlNodePtr top) {
     } else if (IS_XSLT_NAME(cur, "namespace-alias")) {
 	    xsltNamespaceAlias(style, cur);
 	} else {
-	    xsltTransformError(NULL, style, cur,
+            if ((style != NULL) && (style->doc->version != NULL) && (!strncmp(style->doc->version,"1.0",3))) {
+	        xsltTransformError(NULL, style, cur,
+			"xsltParseStylesheetTop: unknown %s element\n",
+			cur->name);
+	        if (style != NULL) style->errors++;
+	    }
+	    else {
+                /* do Forwards-Compatible Processing */
+	        xsltTransformError(NULL, style, cur,
 			"xsltParseStylesheetTop: ignoring unknown %s element\n",
 			cur->name);
-	    if (style != NULL) style->warnings++;
+	        if (style != NULL) style->warnings++;
+            }
 	}
 	cur = cur->next;
     }
@@ -1985,6 +2034,16 @@ xsltParseStylesheetImportedDoc(xmlDocPtr doc, xsltStylesheetPtr style) {
     ret = xsltNewStylesheet();
     if (ret == NULL)
 	return(NULL);
+    if (doc->dict != NULL) {
+        xmlDictFree(ret->dict);
+	ret->dict = doc->dict;
+#ifdef WITH_XSLT_DEBUG
+        xsltGenericDebug(xsltGenericDebugContext,
+                         "reusing dictionary from %s for stylesheet\n",
+			 doc->URL);
+#endif
+	xmlDictReference(ret->dict);
+    }
     
     ret->doc = doc;
     ret->parent = style;	/* needed to prevent loops */
@@ -2067,11 +2126,7 @@ xsltParseStylesheetFile(const xmlChar* filename) {
 	}
     }
 
-#ifdef XSLT_PARSE_OPTIONS
     doc = xmlReadFile((const char *) filename, NULL, XSLT_PARSE_OPTIONS);
-#else
-    doc = xmlParseFile((const char *) filename);
-#endif
     if (doc == NULL) {
 	xsltTransformError(NULL, NULL, NULL,
 		"xsltParseStylesheetFile : cannot parse %s\n", filename);
@@ -2266,14 +2321,18 @@ xsltLoadStylesheetPI(xmlDocPtr doc) {
 		subtree = ID->parent;
 		fake = xmlNewDoc(NULL);
 		if (fake != NULL) {
-#if LIBXML_VERSION >= 20600
                     /*
 		     * the dictionnary should be shared since nodes are
 		     * moved over.
 		     */
 		    fake->dict = doc->dict;
 		    xmlDictReference(doc->dict);
+#ifdef WITH_XSLT_DEBUG
+		    xsltGenericDebug(xsltGenericDebugContext,
+                         "reusing dictionary from %s for stylesheet\n",
+			 doc->URL);
 #endif
+
 		    xmlUnlinkNode(subtree);
 		    xmlAddChild((xmlNodePtr) fake, subtree);
 		    ret = xsltParseStylesheetDoc(fake);
